unit uHlavni;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, IdBaseComponent, IdComponent, IdTCPServer, IdHTTPServer,
  StdCtrls, IniFiles, Menus, xmldom, XMLIntf, msxmldom, XMLDoc;

type
  TfrmHlavni = class(TForm)
    HTTPserver: TIdHTTPServer;
    memStat: TMemo;
    mnuHlavni: TMainMenu;
    mitServer: TMenuItem;
    mitPort: TMenuItem;
    N1: TMenuItem;
    mitDatasoubor: TMenuItem;
    mitChcipni: TMenuItem;
    odlXML: TOpenDialog;
    XMLparser: TXMLDocument;
    mitN2: TMenuItem;
    mitStop: TMenuItem;
    mitStart: TMenuItem;
    procedure HTTPserverCommandGet(AThread: TIdPeerThread;
      RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure mitChcipniClick(Sender: TObject);
    procedure mitServerClick(Sender: TObject);
    procedure mitStartClick(Sender: TObject);
    procedure mitStopClick(Sender: TObject);
    procedure mitDatasouborClick(Sender: TObject);
    procedure mitPortClick(Sender: TObject);
  private
    { Private declarations }
  public
    dataSoubor:string;
    pripPort:integer;
    seznamStanic:THashedStringList;
    seznamStanicString:string;
    procedure NactiNastaveni;
    procedure UlozNastaveni;
    procedure Log(s: string); overload;
    procedure Log(s: string; fmt: array of const); overload;
    function NactiDataSoubor: boolean;
    procedure ServerStop;
    procedure ServerStart;
    function DejSeznamStanic: string;
    { Public declarations }
  end;

var
  frmHlavni: TfrmHlavni;

implementation

{$R *.dfm}

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.HTTPserverCommandGet(AThread: TIdPeerThread;
  RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo);
begin
  Log('Nkdo se sna: '+RequestInfo.Document);
  if lowercase(RequestInfo.Document)='/seznamstanic' then begin
    ResponseInfo.ResponseNo:=200;
    ResponseInfo.ContentType:='text/xml';
    ResponseInfo.ContentText:=SeznamStanicString;
    Log(' - dost o seznam stanic... OK');
  end else if lowercase(RequestInfo.Document)='/najdispojeni' then begin
    ResponseInfo.ResponseNo:=500;
    ResponseInfo.ContentType:='text/plain';
    ResponseInfo.ContentText:='Sorry, lidiky, ale neni to tu...';
    Log(' - dost o vyhledn spojen... SAKRIS');
  end else begin
    ResponseInfo.ResponseNo:=404;
    ResponseInfo.ContentType:='text/plain';
    ResponseInfo.ContentText:='Sorry, lidiky, ale "'+RequestInfo.Document+'" tu nenajdete...';
    Log(' - njak jin ptkrny...');
  end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.NactiNastaveni;
var
  ini:TMemIniFile;
begin
  ini:=TmemIniFile.Create(extractFilePath(paramstr(0))+'nastaveni.ini');
  dataSoubor:=ini.ReadString('server','soubor','data.xml');
  pripPort:=ini.ReadInteger('server','port',80);
  ini.free;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.UlozNastaveni;
var
  ini:TMemIniFile;
begin
  ini:=TmemIniFile.Create(extractFilePath(paramstr(0))+'nastaveni.ini');
  ini.WriteString('server','soubor',dataSoubor);
  ini.WriteInteger('server','port',pripPort);
  ini.UpdateFile;
  ini.free;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.FormCreate(Sender: TObject);
begin
  NactiNastaveni;
  Log('JZServer');
  Log('==========');
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.FormDestroy(Sender: TObject);
begin
  ServerStop;
  seznamStanic.Free;
  UlozNastaveni;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.Log(s: string);
begin
  memStat.Lines.Add(s);
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.Log(s: string; fmt: array of const);
begin
  memStat.Lines.Add(Format(s,fmt));
end;

//******************************************************************************
//******************************************************************************
function TfrmHlavni.NactiDataSoubor: boolean;

  procedure subHledejStaniceVLince(node:IXMLnode);
  var
    i:integer;
    list:IXMLnodeList;
    node2:IXMLnode;
  begin
    list:=node.ChildNodes;
    for i:=0 to list.Count-1 do begin
      node2:=list.Get(i);
      if node2.NodeName='Stanice' then begin
        seznamStanic.Add(node2.NodeValue);
      end;
    end;
  end;
  
  procedure subHledejStanice;
  var
    list:IXMLnodeList;
    i:integer;
  begin
    if seznamStanic=nil then begin
      seznamStanic:=THashedStringList.Create;
      seznamStanic.Sorted:=true;
      seznamStanic.Duplicates:=dupIgnore;
    end;
    seznamStanic.Clear;

    list:=XMLparser.ChildNodes.FindNode('Linky').ChildNodes;
    for i:=0 to list.count-1 do begin
      subHledejStaniceVLince(list.Get(i));
      if (i mod 500) = 0 then begin
        log('Furt hledm... furt ekejte [%d z %d]',[i+1,list.count]);
      end;
    end;
  end;

begin
  result:=true;
  try
    XMLparser.FileName:=dataSoubor;
    XMLparser.Active:=true;
    Log('Hledm stanice... ekejte');
    subHledejStanice;
    Log('Skldm jejich seznam... est trochu ekejte');
    seznamStanicString:=DejSeznamStanic;
    XMLparser.Active:=false;
  except
    result:=false;
  end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitChcipniClick(Sender: TObject);
begin
  Close;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.ServerStop;
begin
  Log('Pprava na sestel...');
  HTTPserver.Active:=false;
  Log('Cha, a u nebm...');
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.ServerStart;
begin
  Log('Nastavuji port %d a napjat ekm, jestli mi to vyjde...',[pripPort]);
  HTTPserver.DefaultPort:=pripPort;
  try
    HTTPserver.Active:=true;
  except
    on e:exception do begin
      Log('... tak to nevylo, protoe %s',[e.message]);
      exit;
    end;
  end;
  Log('...ale jo, vylo');
  Log('Poslouchm na portu %d',[pripPort]);
  Log('Otvrm data soubor "%s"...',[dataSoubor]);
  if not NactiDataSoubor then begin
    Log('Smula, neco se posralo, jdu do kytek...');
    ShowMessage('Smula, neco se posralo, jdu do kytek...');
    application.Terminate;
  end else begin
    Log('V podku, pipraven na nesmysln dotazy...');
  end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitServerClick(Sender: TObject);
begin
  mitStart.Enabled:=not HTTPserver.Active;
  mitStop.Enabled:=HTTPserver.Active;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitStartClick(Sender: TObject);
begin
  ServerStart;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitStopClick(Sender: TObject);
begin
  ServerStop;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitDatasouborClick(Sender: TObject);
begin
  if odlXML.Execute then begin
    ServerStop;
    Log('Nastavuji nov data soubor...');
    dataSoubor:=odlXML.FileName;
    ServerStart;
  end;
end;

//******************************************************************************
//******************************************************************************
procedure TfrmHlavni.mitPortClick(Sender: TObject);
var
  s:string;
  i:integer;
begin
  s:=inttostr(pripPort);
  if InputQuery('JZServer','Zadej nov slo portu',s) then begin
    try
      i:=strToInt(s);
    except
      Log('To je posek, to je posek... tohle neni slo a vy jste lamy!!');
      exit;
    end;
    serverStop;
    pripPort:=i;
    serverStart;
  end;
end;

//******************************************************************************
//******************************************************************************
function TfrmHlavni.DejSeznamStanic: string;
var
  i:integer;
begin
  result:=       '<SeznamStanic>'#13#10;
  for i:=0 to seznamStanic.Count-1 do
    result:=result+#9'<Stanice>'+seznamStanic[i]+'</Stanice>'#13#10;
  result:=result+'</SeznamStanic>';
end;

end.
